const passport = require("passport");
const LocalStrategy = require("passport-local").Strategy;
const JwtStrategy = require("passport-jwt").Strategy;
const ExtractJwt = require("passport-jwt").ExtractJwt;
const bcrypt = require("bcryptjs");
const User = require("../models/User");

// Local Strategy for email/password authentication
passport.use(
	new LocalStrategy(
		{
			usernameField: "email",
			passwordField: "password",
		},
		async (email, password, done) => {
			try {
				// Find user by email
				const user = await User.findByEmail(email);

				if (!user) {
					return done(null, false, { message: "Invalid email or password" });
				}

				// Check if account is locked
				if (
					user.account_locked_until &&
					new Date() < user.account_locked_until
				) {
					return done(null, false, {
						message:
							"Account temporarily locked due to multiple failed login attempts",
					});
				}

				// Check if account is active
				if (!user.is_active) {
					return done(null, false, { message: "Account has been deactivated" });
				}

				// Check if email is verified (for non-admin users)
				if (!user.is_verified && user.role !== "super_admin") {
					return done(null, false, {
						message: "Please verify your email address before logging in",
					});
				}

				// Verify password
				const isValidPassword = await bcrypt.compare(
					password,
					user.password_hash
				);

				if (!isValidPassword) {
					// Increment failed login attempts
					await User.incrementFailedLoginAttempts(user.id);
					return done(null, false, { message: "Invalid email or password" });
				}

				// Reset failed login attempts and update last login
				await User.resetFailedLoginAttempts(user.id);
				await User.updateLastLogin(user.id);

				// Remove sensitive information before returning user
				const userSafe = {
					id: user.id,
					uuid: user.uuid,
					email: user.email,
					first_name: user.first_name,
					last_name: user.last_name,
					role: user.role,
					is_verified: user.is_verified,
					is_active: user.is_active,
					created_at: user.created_at,
					last_login: user.last_login,
					must_change_password: user.must_change_password,
				};

				return done(null, userSafe);
			} catch (error) {
				return done(error);
			}
		}
	)
);

// JWT Strategy for token-based authentication
passport.use(
	new JwtStrategy(
		{
			jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
			secretOrKey: process.env.JWT_SECRET || "trhrtjtzmkjt56fgdfg3tcvv",
			issuer: process.env.JWT_ISSUER || "formies",
			audience: process.env.JWT_AUDIENCE || "formies-users",
		},
		async (payload, done) => {
			try {
				// Check if token is blacklisted
				const isBlacklisted = await User.isTokenBlacklisted(payload.jti);
				if (isBlacklisted) {
					return done(null, false, { message: "Token has been revoked" });
				}

				// Find user by ID
				const user = await User.findById(payload.sub);

				if (!user) {
					return done(null, false, { message: "User not found" });
				}

				// Check if account is active
				if (!user.is_active) {
					return done(null, false, { message: "Account has been deactivated" });
				}

				// Remove sensitive information before returning user
				const userSafe = {
					id: user.id,
					uuid: user.uuid,
					email: user.email,
					first_name: user.first_name,
					last_name: user.last_name,
					role: user.role,
					is_verified: user.is_verified,
					is_active: user.is_active,
					created_at: user.created_at,
					last_login: user.last_login,
					must_change_password: user.must_change_password,
				};

				return done(null, userSafe);
			} catch (error) {
				return done(error);
			}
		}
	)
);

// Serialize user for session
passport.serializeUser((user, done) => {
	done(null, user.id);
});

// Deserialize user from session
passport.deserializeUser(async (id, done) => {
	try {
		const user = await User.findById(id);
		if (user) {
			const userSafe = {
				id: user.id,
				uuid: user.uuid,
				email: user.email,
				first_name: user.first_name,
				last_name: user.last_name,
				role: user.role,
				is_verified: user.is_verified,
				is_active: user.is_active,
				created_at: user.created_at,
				last_login: user.last_login,
				must_change_password: user.must_change_password,
			};
			done(null, userSafe);
		} else {
			done(null, false);
		}
	} catch (error) {
		done(error);
	}
});

module.exports = passport;
