const { createClient } = require("redis");

let redisClient = null;
let connectionAttempted = false;
let isRedisAvailable = false;

const connectRedis = async () => {
	if (redisClient) {
		return redisClient;
	}

	// If we already tried and failed, don't keep trying
	if (connectionAttempted && !isRedisAvailable) {
		return null;
	}

	connectionAttempted = true;

	const redisHost = process.env.REDIS_HOST || "localhost";
	const redisPort = process.env.REDIS_PORT || 6379;
	const redisPassword = process.env.REDIS_PASSWORD || "";

	const config = {
		socket: {
			host: redisHost,
			port: redisPort,
			connectTimeout: 1000, // Reduced timeout to 1 second
			lazyConnect: true,
		},
		// Disable automatic reconnection to prevent spam
		retry_unfulfilled_commands: false,
		enable_offline_queue: false,
	};

	// Add password if provided
	if (redisPassword) {
		config.password = redisPassword;
	}

	redisClient = createClient(config);

	// Only log the first error, not subsequent ones
	let errorLogged = false;
	redisClient.on("error", (err) => {
		if (!errorLogged) {
			console.warn("Redis connection failed:", err.code || err.message);
			console.warn("Falling back to in-memory rate limiting");
			errorLogged = true;
		}
		isRedisAvailable = false;
	});

	redisClient.on("connect", () => {
		console.log("Connected to Redis");
		isRedisAvailable = true;
	});

	redisClient.on("disconnect", () => {
		if (isRedisAvailable) {
			console.log("Disconnected from Redis");
		}
		isRedisAvailable = false;
	});

	try {
		await redisClient.connect();
		console.log("Redis client connected successfully");
		isRedisAvailable = true;
	} catch (error) {
		console.warn("Failed to connect to Redis:", error.code || error.message);
		console.warn("Continuing with in-memory rate limiting");
		isRedisAvailable = false;
		redisClient = null;
		return null;
	}

	return redisClient;
};

const getRedisClient = () => {
	if (!redisClient || !isRedisAvailable) {
		throw new Error("Redis client not available");
	}
	return redisClient;
};

const closeRedis = async () => {
	if (redisClient && isRedisAvailable) {
		try {
			await redisClient.quit();
			console.log("Redis connection closed");
		} catch (error) {
			// Ignore errors during shutdown
		}
	}
	redisClient = null;
	isRedisAvailable = false;
	connectionAttempted = false;
};

const isRedisConnected = () => {
	return isRedisAvailable && redisClient && redisClient.isOpen;
};

module.exports = {
	connectRedis,
	getRedisClient,
	closeRedis,
	isRedisConnected,
};
