const express = require("express");
const passport = require("../config/passport");
const User = require("../models/User");
const jwtService = require("../services/jwtService");
const emailService = require("../services/emailService");
const { body } = require("express-validator");
const {
	validateRegistration,
	validateLogin,
	validateForgotPassword,
	validateResetPassword,
	validateProfileUpdate,
	handleValidationErrors,
} = require("../middleware/validation");
const {
	authRateLimit,
	passwordResetRateLimit,
	registrationRateLimit,
	requireAuth,
	requireVerifiedAuth,
} = require("../middleware/authMiddleware");

const router = express.Router();

// Register new user
router.post(
	"/register",
	registrationRateLimit,
	validateRegistration,
	async (req, res) => {
		try {
			const { email, password, first_name, last_name } = req.body;

			// Check if user already exists
			const existingUser = await User.findByEmail(email);
			if (existingUser) {
				return res.status(409).json({
					success: false,
					message: "An account with this email address already exists",
				});
			}

			// Create new user
			const newUser = await User.create({
				email,
				password,
				first_name,
				last_name,
				role: "user",
				is_verified: false,
			});

			// Send verification email
			if (emailService.isAvailable()) {
				await emailService.sendVerificationEmail(
					newUser.email,
					newUser.first_name,
					newUser.verification_token
				);
			}

			res.status(201).json({
				success: true,
				message:
					"Account created successfully. Please check your email to verify your account.",
				data: {
					user: {
						id: newUser.id,
						uuid: newUser.uuid,
						email: newUser.email,
						first_name: newUser.first_name,
						last_name: newUser.last_name,
						is_verified: newUser.is_verified,
					},
				},
			});
		} catch (error) {
			console.error("Registration error:", error);
			res.status(500).json({
				success: false,
				message: error.message || "Registration failed",
			});
		}
	}
);

// Login user
router.post("/login", authRateLimit, validateLogin, (req, res, next) => {
	passport.authenticate(
		"local",
		{ session: false },
		async (err, user, info) => {
			try {
				if (err) {
					return res.status(500).json({
						success: false,
						message: "Authentication error",
						error: err.message,
					});
				}

				if (!user) {
					return res.status(401).json({
						success: false,
						message: info.message || "Invalid credentials",
					});
				}

				// Check if password change is required
				if (user.must_change_password) {
					// Generate a temporary token that's only valid for password change
					// This step depends on how you want to handle the forced change flow.
					// For now, we'll just send a specific response.
					// A more robust solution might involve a temporary, restricted token.
					return res.status(403).json({
						// 403 Forbidden, but with a specific reason
						success: false,
						message: "Password change required.",
						code: "MUST_CHANGE_PASSWORD",
						data: {
							user: {
								// Send minimal user info
								id: user.id,
								uuid: user.uuid,
								email: user.email,
								role: user.role,
							},
						},
					});
				}

				// Generate JWT tokens
				const sessionInfo = {
					userAgent: req.get("User-Agent"),
					ipAddress: req.ip,
				};

				const tokens = jwtService.generateTokenPair(user, sessionInfo);

				res.json({
					success: true,
					message: "Login successful",
					data: {
						user: {
							id: user.id,
							uuid: user.uuid,
							email: user.email,
							first_name: user.first_name,
							last_name: user.last_name,
							role: user.role,
							is_verified: user.is_verified,
							last_login: user.last_login,
						},
						...tokens,
					},
				});
			} catch (error) {
				console.error("Login error:", error);
				res.status(500).json({
					success: false,
					message: "Login failed",
				});
			}
		}
	)(req, res, next);
});

// Refresh access token
router.post("/refresh", async (req, res) => {
	try {
		const { refreshToken } = req.body;

		if (!refreshToken) {
			return res.status(400).json({
				success: false,
				message: "Refresh token is required",
			});
		}

		const sessionInfo = {
			userAgent: req.get("User-Agent"),
			ipAddress: req.ip,
		};

		const result = await jwtService.refreshAccessToken(
			refreshToken,
			sessionInfo
		);

		res.json({
			success: true,
			message: "Token refreshed successfully",
			data: result,
		});
	} catch (error) {
		console.error("Token refresh error:", error);
		res.status(401).json({
			success: false,
			message: error.message || "Token refresh failed",
		});
	}
});

// Logout user
router.post("/logout", requireAuth, async (req, res) => {
	try {
		const authHeader = req.headers.authorization;
		const token = jwtService.extractTokenFromHeader(authHeader);

		if (token) {
			await jwtService.revokeToken(token);
		}

		res.json({
			success: true,
			message: "Logged out successfully",
		});
	} catch (error) {
		console.error("Logout error:", error);
		res.status(500).json({
			success: false,
			message: "Logout failed",
		});
	}
});

// Logout from all devices
router.post("/logout-all", requireAuth, async (req, res) => {
	try {
		const revokedCount = await jwtService.revokeAllUserTokens(req.user.id);

		res.json({
			success: true,
			message: `Logged out from ${revokedCount} devices successfully`,
		});
	} catch (error) {
		console.error("Logout all error:", error);
		res.status(500).json({
			success: false,
			message: "Logout from all devices failed",
		});
	}
});

// Verify email
router.get("/verify-email", async (req, res) => {
	try {
		const { token } = req.query;

		if (!token) {
			return res.status(400).json({
				success: false,
				message: "Verification token is required",
			});
		}

		const user = await User.findByVerificationToken(token);
		if (!user) {
			return res.status(400).json({
				success: false,
				message: "Invalid or expired verification token",
			});
		}

		if (user.is_verified) {
			return res.status(400).json({
				success: false,
				message: "Email is already verified",
			});
		}

		const verified = await User.verifyEmail(token);
		if (!verified) {
			return res.status(400).json({
				success: false,
				message: "Email verification failed",
			});
		}

		// Send welcome email
		if (emailService.isAvailable()) {
			await emailService.sendWelcomeEmail(user.email, user.first_name);
		}

		res.json({
			success: true,
			message: "Email verified successfully! You can now access all features.",
		});
	} catch (error) {
		console.error("Email verification error:", error);
		res.status(500).json({
			success: false,
			message: "Email verification failed",
		});
	}
});

// Resend verification email
router.post("/resend-verification", authRateLimit, async (req, res) => {
	try {
		const { email } = req.body;

		if (!email) {
			return res.status(400).json({
				success: false,
				message: "Email is required",
			});
		}

		const user = await User.findByEmail(email);
		if (!user) {
			// Don't reveal if email exists or not
			return res.json({
				success: true,
				message:
					"If an account with this email exists and is not verified, a verification email has been sent.",
			});
		}

		if (user.is_verified) {
			return res.status(400).json({
				success: false,
				message: "Email is already verified",
			});
		}

		// Send verification email
		if (emailService.isAvailable() && user.verification_token) {
			await emailService.sendVerificationEmail(
				user.email,
				user.first_name,
				user.verification_token
			);
		}

		res.json({
			success: true,
			message:
				"If an account with this email exists and is not verified, a verification email has been sent.",
		});
	} catch (error) {
		console.error("Resend verification error:", error);
		res.status(500).json({
			success: false,
			message: "Failed to resend verification email",
		});
	}
});

// Forgot password - Request password reset
router.post(
	"/forgot-password",
	passwordResetRateLimit,
	validateForgotPassword,
	async (req, res) => {
		try {
			const { email } = req.body;

			// Don't reveal if email exists or not for security
			const user = await User.findByEmail(email);

			if (user) {
				// Generate password reset token
				const resetData = await User.setPasswordResetToken(email);

				if (resetData && emailService.isAvailable()) {
					await emailService.sendPasswordResetEmail(
						user.email,
						user.first_name,
						resetData.token
					);
				}
			}

			// Always return success to prevent email enumeration
			res.json({
				success: true,
				message:
					"If an account with this email exists, a password reset email has been sent.",
			});
		} catch (error) {
			console.error("Forgot password error:", error);
			res.status(500).json({
				success: false,
				message: "Failed to process password reset request",
			});
		}
	}
);

// Reset password - Change password using reset token
router.post(
	"/reset-password",
	passwordResetRateLimit,
	validateResetPassword,
	async (req, res) => {
		try {
			const { token, password } = req.body;

			// Find user by reset token
			const user = await User.findByPasswordResetToken(token);
			if (!user) {
				return res.status(400).json({
					success: false,
					message: "Invalid or expired reset token",
				});
			}

			// Update password
			const updated = await User.updatePassword(user.id, password);
			if (!updated) {
				return res.status(500).json({
					success: false,
					message: "Failed to update password",
				});
			}

			// Send password changed notification
			if (emailService.isAvailable()) {
				await emailService.sendPasswordChangedEmail(
					user.email,
					user.first_name
				);
			}

			// Revoke all existing sessions for security
			await jwtService.revokeAllUserTokens(user.id);

			res.json({
				success: true,
				message:
					"Password has been reset successfully. Please log in with your new password.",
			});
		} catch (error) {
			console.error("Reset password error:", error);
			res.status(500).json({
				success: false,
				message: "Failed to reset password",
			});
		}
	}
);

// Get current user profile
router.get("/profile", requireAuth, async (req, res) => {
	try {
		const stats = await User.getUserStats(req.user.id);

		res.json({
			success: true,
			data: {
				user: {
					...req.user,
					stats,
				},
			},
		});
	} catch (error) {
		console.error("Profile fetch error:", error);
		res.status(500).json({
			success: false,
			message: "Failed to fetch profile",
		});
	}
});

// Update user profile
router.put("/profile", requireAuth, validateProfileUpdate, async (req, res) => {
	try {
		const { first_name, last_name, email } = req.body;
		const updates = {};

		if (first_name !== undefined) updates.first_name = first_name;
		if (last_name !== undefined) updates.last_name = last_name;
		if (email !== undefined && email !== req.user.email) {
			updates.email = email;
			// If email is being changed, user needs to verify the new email
			// For now, we'll just update it directly
		}

		if (Object.keys(updates).length === 0) {
			return res.status(400).json({
				success: false,
				message: "No valid fields to update",
			});
		}

		const updated = await User.updateProfile(req.user.id, updates);
		if (!updated) {
			return res.status(400).json({
				success: false,
				message: "Profile update failed",
			});
		}

		// Get updated user data
		const updatedUser = await User.findById(req.user.id);

		res.json({
			success: true,
			message: "Profile updated successfully",
			data: {
				user: {
					id: updatedUser.id,
					uuid: updatedUser.uuid,
					email: updatedUser.email,
					first_name: updatedUser.first_name,
					last_name: updatedUser.last_name,
					role: updatedUser.role,
					is_verified: updatedUser.is_verified,
					is_active: updatedUser.is_active,
				},
			},
		});
	} catch (error) {
		console.error("Profile update error:", error);
		res.status(500).json({
			success: false,
			message: error.message || "Profile update failed",
		});
	}
});

// Get user's active sessions
router.get("/sessions", requireAuth, async (req, res) => {
	try {
		const sessions = await User.getUserActiveSessions(req.user.id);

		res.json({
			success: true,
			data: {
				sessions,
			},
		});
	} catch (error) {
		console.error("Get sessions error:", error);
		res.status(500).json({
			success: false,
			message: "Failed to fetch sessions",
		});
	}
});

// Revoke a specific session
router.delete("/sessions/:jti", requireAuth, async (req, res) => {
	try {
		const { jti } = req.params;

		// Verify the session belongs to the user
		const session = await User.getSessionByJti(jti);
		if (!session || session.user_id !== req.user.id) {
			return res.status(404).json({
				success: false,
				message: "Session not found",
			});
		}

		const revoked = await User.revokeSession(jti);
		if (!revoked) {
			return res.status(500).json({
				success: false,
				message: "Failed to revoke session",
			});
		}

		res.json({
			success: true,
			message: "Session revoked successfully",
		});
	} catch (error) {
		console.error("Revoke session error:", error);
		res.status(500).json({
			success: false,
			message: "Failed to revoke session",
		});
	}
});

// Get current session information
router.get("/current-session", requireAuth, async (req, res) => {
	try {
		const authHeader = req.headers.authorization;
		const token = jwtService.extractTokenFromHeader(authHeader);

		if (!token) {
			return res.status(401).json({
				success: false,
				message: "No token provided",
			});
		}

		const session = await jwtService.getCurrentSession(token);

		res.json({
			success: true,
			data: {
				session,
			},
		});
	} catch (error) {
		console.error("Get current session error:", error);
		res.status(500).json({
			success: false,
			message: "Failed to get current session information",
		});
	}
});

// Change password for logged-in users
router.put(
	"/change-password",
	requireAuth,
	[
		body("currentPassword")
			.notEmpty()
			.withMessage("Current password is required"),
		body("newPassword")
			.isLength({ min: 8 })
			.withMessage("New password must be at least 8 characters long")
			.matches(
				/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]/
			)
			.withMessage(
				"New password must contain at least one lowercase letter, one uppercase letter, one number, and one special character"
			),
		body("confirmNewPassword").custom((value, { req }) => {
			if (value !== req.body.newPassword) {
				throw new Error("Password confirmation does not match new password");
			}
			return true;
		}),
		handleValidationErrors,
	],
	async (req, res) => {
		try {
			const { currentPassword, newPassword } = req.body;
			const bcrypt = require("bcryptjs");

			// Get user with password hash
			const user = await User.findById(req.user.id);
			if (!user) {
				return res.status(404).json({
					success: false,
					message: "User not found",
				});
			}

			// Verify current password
			const isCurrentPasswordValid = await bcrypt.compare(
				currentPassword,
				user.password_hash
			);
			if (!isCurrentPasswordValid) {
				return res.status(400).json({
					success: false,
					message: "Current password is incorrect",
				});
			}

			// Update password
			const updated = await User.updatePassword(user.id, newPassword);
			if (!updated) {
				return res.status(500).json({
					success: false,
					message: "Failed to update password",
				});
			}

			// Send password changed notification
			if (emailService.isAvailable()) {
				await emailService.sendPasswordChangedEmail(
					user.email,
					user.first_name
				);
			}

			// Revoke all other sessions (keep current session)
			const authHeader = req.headers.authorization;
			const currentToken = jwtService.extractTokenFromHeader(authHeader);
			const decoded = jwtService.verifyToken(currentToken);

			// Revoke all sessions except current one
			await jwtService.revokeAllUserTokensExcept(user.id, decoded.jti);

			res.json({
				success: true,
				message: "Password changed successfully",
			});
		} catch (error) {
			console.error("Change password error:", error);
			res.status(500).json({
				success: false,
				message: "Failed to change password",
			});
		}
	}
);

// Force password change if must_change_password is true
router.post(
	"/force-change-password",
	requireAuth, // Ensures user is logged in (even if with must_change_password = true)
	[
		body("newPassword")
			.isLength({ min: 8 })
			.withMessage("Password must be at least 8 characters long"),
	],
	handleValidationErrors,
	async (req, res) => {
		try {
			const { newPassword } = req.body;
			const userId = req.user.id;

			// Double check if user still needs to change password
			// (req.user might be from a valid token but DB state could have changed)
			const currentUser = await User.findById(userId);
			if (!currentUser || !currentUser.must_change_password) {
				return res.status(400).json({
					success: false,
					message: "Password change not required or user not found.",
				});
			}

			// Update password and clear the flag
			const updated = await User.updatePasswordAndClearChangeFlag(
				userId,
				newPassword
			);

			if (!updated) {
				return res.status(500).json({
					success: false,
					message: "Failed to update password.",
				});
			}

			// Log out all other sessions for this user for security
			const authHeader = req.headers.authorization;
			const currentToken = jwtService.extractTokenFromHeader(authHeader);
			const decoded = jwtService.verifyToken(currentToken); // Make sure verifyToken doesn't throw on expired/invalid for this flow if needed or handle it

			if (decoded && decoded.jti) {
				// Ensure there is a jti in the current token
				await jwtService.revokeAllUserTokensExcept(userId, decoded.jti);
			} else {
				// Fallback if current token has no jti, revoke all including current. User will need to log in again.
				await jwtService.revokeAllUserTokens(userId);
			}

			res.json({
				success: true,
				message:
					"Password changed successfully. Please log in again with your new password.",
			});
		} catch (error) {
			console.error("Force change password error:", error);
			res.status(500).json({
				success: false,
				message: "Failed to change password",
			});
		}
	}
);

module.exports = router;
