const express = require("express");
const pool = require("../config/database"); // Assuming database config is here
const { requireAuth } = require("../middleware/authMiddleware"); // Assuming auth middleware
const { v4: uuidv4 } = require("uuid"); // Make sure to require uuid
const { sendNtfyNotification } = require("../services/notification"); // Fixed import path
const {
	generateApiKeyParts,
	hashApiKeySecret,
} = require("../utils/apiKeyHelper.js"); // Import API key helpers

const router = express.Router();

// All dashboard routes require authentication
router.use(requireAuth);

// GET /dashboard - Main dashboard view (My Forms)
router.get("/", async (req, res) => {
	try {
		const [forms] = await pool.query(
			`SELECT f.uuid, f.name, f.created_at, f.is_archived,
            (SELECT COUNT(*) FROM submissions WHERE form_uuid = f.uuid) as submission_count
            FROM forms f 
            WHERE f.user_id = ? 
            ORDER BY f.created_at DESC`,
			[req.user.id]
		);

		res.render("dashboard", {
			user: req.user,
			forms: forms,
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "my_forms", // To tell dashboard.ejs which section to show
			pageTitle: "My Forms",
		});
	} catch (error) {
		console.error("Error fetching user forms:", error);
		// res.status(500).send("Error fetching forms"); // Or render an error page
		res.render("dashboard", {
			user: req.user,
			forms: [],
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "my_forms",
			pageTitle: "My Forms",
			error: "Could not load your forms at this time.",
		});
	}
});

// GET /dashboard/create-form - Display page to create a new form
router.get("/create-form", (req, res) => {
	res.render("dashboard", {
		user: req.user,
		appUrl: `${req.protocol}://${req.get("host")}`,
		view: "create_form", // To tell dashboard.ejs to show the create form section
		pageTitle: "Create New Form",
	});
});

// POST /dashboard/forms/create - Handle new form creation
router.post("/forms/create", async (req, res) => {
	const formName = req.body.formName || "Untitled Form";
	const newUuid = uuidv4();
	try {
		await pool.query(
			"INSERT INTO forms (uuid, name, user_id) VALUES (?, ?, ?)",
			[newUuid, formName, req.user.id]
		);
		console.log(
			`Form created: ${formName} with UUID: ${newUuid} for user: ${req.user.id}`
		);

		// Optional: Send a notification (if your ntfy setup is user-specific or global)
		// Consider if this notification is still relevant or needs adjustment for user context
		if (process.env.NTFY_ENABLED === "true" && process.env.NTFY_TOPIC_URL) {
			try {
				await sendNtfyNotification(
					"New Form Created (User)",
					`Form \"${formName}\" (UUID: ${newUuid}) was created by user ${req.user.email}.`,
					"high"
				);
			} catch (ntfyError) {
				console.error(
					"Failed to send ntfy notification for new form creation:",
					ntfyError
				);
			}
		}

		res.redirect("/dashboard"); // Redirect to the user's form list
	} catch (error) {
		console.error("Error creating form for user:", error);
		// Render the create form page again with an error message
		res.render("dashboard", {
			user: req.user,
			appUrl: `${req.protocol}://${req.get("host")}`,
			view: "create_form",
			pageTitle: "Create New Form",
			error: "Failed to create form. Please try again.",
			formNameValue: formName, // Pass back the entered form name
		});
	}
});

// GET /dashboard/submissions/:formUuid - View submissions for a specific form
router.get("/submissions/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	const page = parseInt(req.query.page) || 1;
	const limit = parseInt(req.query.limit) || 10; // Default 10 submissions per page
	const offset = (page - 1) * limit;

	try {
		// First, verify the user owns the form
		const [formDetails] = await pool.query(
			"SELECT name, user_id FROM forms WHERE uuid = ?",
			[formUuid]
		);

		if (formDetails.length === 0) {
			// return res.status(404).send("Form not found.");
			return res.render("dashboard", {
				user: req.user,
				view: "my_forms", // Redirect to a safe place or show a specific error view
				pageTitle: "Form Not Found",
				error: "The form you are looking for does not exist.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [], // Provide empty forms array if redirecting to my_forms with an error
			});
		}

		if (formDetails[0].user_id !== req.user.id) {
			// return res.status(403).send("Access denied. You do not own this form.");
			return res.render("dashboard", {
				user: req.user,
				view: "my_forms", // Redirect to a safe place or show a specific error view
				pageTitle: "Access Denied",
				error: "You do not have permission to view submissions for this form.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [], // Provide empty forms array
			});
		}

		const formName = formDetails[0].name;

		// Get total count of submissions for pagination
		const [countResult] = await pool.query(
			"SELECT COUNT(*) as total FROM submissions WHERE form_uuid = ?",
			[formUuid]
		);
		const totalSubmissions = countResult[0].total;
		const totalPages = Math.ceil(totalSubmissions / limit);

		// Fetch paginated submissions
		const [submissions] = await pool.query(
			"SELECT id, data, ip_address, submitted_at FROM submissions WHERE form_uuid = ? ORDER BY submitted_at DESC LIMIT ? OFFSET ?",
			[formUuid, limit, offset]
		);

		res.render("dashboard", {
			user: req.user,
			view: "form_submissions",
			pageTitle: `Submissions for ${formName}`,
			submissions: submissions,
			formUuid: formUuid,
			formName: formName,
			appUrl: `${req.protocol}://${req.get("host")}`,
			pagination: {
				currentPage: page,
				totalPages: totalPages,
				totalSubmissions: totalSubmissions,
				limit: limit,
			},
		});
	} catch (error) {
		console.error(
			"Error fetching submissions for form:",
			formUuid,
			"user:",
			req.user.id,
			error
		);
		// Render an error state within the dashboard
		res.render("dashboard", {
			user: req.user,
			view: "form_submissions", // Or a dedicated error view component
			pageTitle: "Error Loading Submissions",
			error:
				"Could not load submissions for this form. Please try again later.",
			formUuid: formUuid,
			formName: "Error", // Placeholder for formName when an error occurs
			submissions: [],
			appUrl: `${req.protocol}://${req.get("host")}`,
			pagination: {
				currentPage: 1,
				totalPages: 1,
				totalSubmissions: 0,
				limit: limit,
			},
		});
	}
});

// GET /dashboard/submissions/:formUuid/export - Export submissions to CSV
router.get("/submissions/:formUuid/export", async (req, res) => {
	const { formUuid } = req.params;
	try {
		// First, verify the user owns the form
		const [formDetails] = await pool.query(
			"SELECT name, user_id FROM forms WHERE uuid = ?",
			[formUuid]
		);

		if (formDetails.length === 0) {
			return res.status(404).send("Form not found.");
		}

		if (formDetails[0].user_id !== req.user.id) {
			return res.status(403).send("Access denied. You do not own this form.");
		}
		const formName = formDetails[0].name;

		const [submissions] = await pool.query(
			"SELECT data, ip_address, submitted_at FROM submissions WHERE form_uuid = ? ORDER BY submitted_at DESC",
			[formUuid]
		);

		// Create CSV content
		const headers = ["Submitted At", "IP Address"];
		const rows = submissions.map((submission) => {
			const data = JSON.parse(submission.data);
			// Add all form fields as headers
			Object.keys(data).forEach((key) => {
				if (!headers.includes(key)) {
					headers.push(key);
				}
			});
			return {
				submitted_at: new Date(submission.submitted_at).toISOString(),
				ip_address: submission.ip_address,
				...data,
			};
		});

		// Generate CSV content
		let csvContent = headers.join(",") + "\n";
		rows.forEach((row) => {
			const values = headers.map((header) => {
				const value = row[header] || "";
				// Escape commas and quotes in values
				return `"${String(value).replace(/"/g, '""')}"`;
			});
			csvContent += values.join(",") + "\n";
		});

		// Set response headers for CSV download
		res.setHeader("Content-Type", "text/csv");
		res.setHeader(
			"Content-Disposition",
			`attachment; filename="${formName}-submissions.csv"`
		);
		res.send(csvContent);
	} catch (error) {
		console.error(
			"Error exporting submissions:",
			formUuid,
			"user:",
			req.user.id,
			error
		);
		res.status(500).send("Error exporting submissions");
	}
});

// GET /dashboard/forms/:formUuid/settings - Display form settings page
router.get("/forms/:formUuid/settings", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const [formDetailsArray] = await pool.query(
			"SELECT name, user_id, email_notifications_enabled, notification_email_address, recaptcha_enabled, thank_you_url, thank_you_message, allowed_domains FROM forms WHERE uuid = ?",
			[formUuid]
		);

		if (formDetailsArray.length === 0) {
			return res.render("dashboard", {
				user: req.user,
				view: "my_forms",
				pageTitle: "Form Not Found",
				error: "The form you are trying to access settings for does not exist.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [],
			});
		}
		const formDetails = formDetailsArray[0];

		if (formDetails.user_id !== req.user.id) {
			return res.render("dashboard", {
				user: req.user,
				view: "my_forms",
				pageTitle: "Access Denied",
				error: "You do not have permission to access settings for this form.",
				appUrl: `${req.protocol}://${req.get("host")}`,
				forms: [],
			});
		}

		res.render("dashboard", {
			user: req.user,
			view: "form_settings",
			pageTitle: `Settings for ${formDetails.name}`,
			formName: formDetails.name, // For the header
			currentFormName: formDetails.name, // For the input field value
			formUuid: formUuid,
			currentEmailNotificationsEnabled: formDetails.email_notifications_enabled,
			currentNotificationEmailAddress: formDetails.notification_email_address,
			currentRecaptchaEnabled: formDetails.recaptcha_enabled,
			currentThankYouUrl: formDetails.thank_you_url,
			currentThankYouMessage: formDetails.thank_you_message,
			currentAllowedDomains: formDetails.allowed_domains,
			appUrl: `${req.protocol}://${req.get("host")}`,
			successMessage: req.query.successMessage,
			errorMessage: req.query.errorMessage,
		});
	} catch (error) {
		console.error(
			"Error fetching form settings for form:",
			formUuid,
			"user:",
			req.user.id,
			error
		);
		res.render("dashboard", {
			user: req.user,
			view: "my_forms",
			pageTitle: "Error",
			error: "Could not load settings for this form. Please try again later.",
			appUrl: `${req.protocol}://${req.get("host")}`,
			forms: [], // Go back to a safe page
		});
	}
});

// POST /dashboard/forms/:formUuid/settings/update - Update various form settings
router.post("/forms/:formUuid/settings/update", async (req, res) => {
	const { formUuid } = req.params;
	const {
		formName,
		emailNotificationsEnabled,
		notificationEmailAddress,
		recaptchaEnabled,
		thankYouUrl,
		thankYouMessage,
		allowedDomains,
	} = req.body;

	// Validate formName (must not be empty if provided)
	if (formName !== undefined && formName.trim() === "") {
		return res.redirect(
			`/dashboard/forms/${formUuid}/settings?errorMessage=Form name cannot be empty.`
		);
	}

	// Convert checkbox values which might come as 'on' or undefined
	const finalEmailNotificationsEnabled =
		emailNotificationsEnabled === "on" || emailNotificationsEnabled === true;
	const finalRecaptchaEnabled =
		recaptchaEnabled === "on" || recaptchaEnabled === true;

	// If email notifications are enabled, but no specific address is provided,
	// and there's no existing specific address, we might want to clear it or use user's default.
	// For now, if it's blank, we'll store NULL or an empty string based on DB.
	// Let's assume an empty string means "use user's default email" when sending.
	const finalNotificationEmailAddress = notificationEmailAddress
		? notificationEmailAddress.trim()
		: null;

	try {
		// First, verify the user owns the form
		const [formOwnerCheck] = await pool.query(
			"SELECT user_id FROM forms WHERE uuid = ?",
			[formUuid]
		);

		if (
			formOwnerCheck.length === 0 ||
			formOwnerCheck[0].user_id !== req.user.id
		) {
			// Security: Do not reveal if form exists or not, just deny.
			// Or redirect to a generic error page/dashboard.
			// For now, let's redirect with a generic error.
			return res.redirect(
				`/dashboard/forms/${formUuid}/settings?errorMessage=Access denied or form not found.`
			);
		}

		// Build the update query dynamically based on which fields are provided
		const updates = {};
		if (formName !== undefined) updates.name = formName.trim();
		if (emailNotificationsEnabled !== undefined)
			updates.email_notifications_enabled = finalEmailNotificationsEnabled;
		if (notificationEmailAddress !== undefined)
			updates.notification_email_address = finalNotificationEmailAddress; // Allows clearing the address
		if (recaptchaEnabled !== undefined)
			updates.recaptcha_enabled = finalRecaptchaEnabled;
		if (thankYouUrl !== undefined)
			updates.thank_you_url = thankYouUrl.trim() || null;
		if (thankYouMessage !== undefined)
			updates.thank_you_message = thankYouMessage.trim() || null;
		if (allowedDomains !== undefined)
			updates.allowed_domains = allowedDomains.trim() || null;

		if (Object.keys(updates).length === 0) {
			// Nothing to update, redirect back, maybe with an info message
			return res.redirect(
				`/dashboard/forms/${formUuid}/settings?successMessage=No changes were made.`
			);
		}

		updates.updated_at = new Date(); // Explicitly set updated_at

		await pool.query("UPDATE forms SET ? WHERE uuid = ? AND user_id = ?", [
			updates,
			formUuid,
			req.user.id, // Ensure user_id match as an extra precaution
		]);

		console.log(
			`Form settings updated for ${formUuid} by user ${req.user.id}:`,
			updates
		);
		res.redirect(
			`/dashboard/forms/${formUuid}/settings?successMessage=Settings updated successfully!`
		);
	} catch (error) {
		console.error(
			"Error updating form settings for form:",
			formUuid,
			"user:",
			req.user.id,
			error
		);
		res.redirect(
			`/dashboard/forms/${formUuid}/settings?errorMessage=Error updating settings. Please try again.`
		);
	}
});

// POST /dashboard/forms/archive/:formUuid - Archive a form
router.post("/forms/archive/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const [formDetails] = await pool.query(
			"SELECT user_id FROM forms WHERE uuid = ?",
			[formUuid]
		);
		if (formDetails.length === 0) {
			return res.redirect(
				"/dashboard?errorMessage=" + encodeURIComponent("Form not found.")
			);
		}
		if (formDetails[0].user_id !== req.user.id) {
			return res.redirect(
				"/dashboard?errorMessage=" +
					encodeURIComponent("You do not have permission to modify this form.")
			);
		}

		await pool.query(
			"UPDATE forms SET is_archived = true WHERE uuid = ? AND user_id = ?",
			[formUuid, req.user.id]
		);
		res.redirect(
			"/dashboard?successMessage=" +
				encodeURIComponent("Form archived successfully.")
		);
	} catch (error) {
		console.error("Error archiving form:", formUuid, error);
		res.redirect(
			"/dashboard?errorMessage=" + encodeURIComponent("Failed to archive form.")
		);
	}
});

// POST /dashboard/forms/unarchive/:formUuid - Unarchive a form
router.post("/forms/unarchive/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	try {
		const [formDetails] = await pool.query(
			"SELECT user_id FROM forms WHERE uuid = ?",
			[formUuid]
		);
		if (formDetails.length === 0) {
			return res.redirect(
				"/dashboard?errorMessage=" + encodeURIComponent("Form not found.")
			);
		}
		if (formDetails[0].user_id !== req.user.id) {
			return res.redirect(
				"/dashboard?errorMessage=" +
					encodeURIComponent("You do not have permission to modify this form.")
			);
		}

		await pool.query(
			"UPDATE forms SET is_archived = false WHERE uuid = ? AND user_id = ?",
			[formUuid, req.user.id]
		);
		res.redirect(
			"/dashboard?successMessage=" +
				encodeURIComponent("Form unarchived successfully.")
		);
	} catch (error) {
		console.error("Error unarchiving form:", formUuid, error);
		res.redirect(
			"/dashboard?errorMessage=" +
				encodeURIComponent("Failed to unarchive form.")
		);
	}
});

// POST /dashboard/forms/delete/:formUuid - Permanently delete a form
router.post("/forms/delete/:formUuid", async (req, res) => {
	const { formUuid } = req.params;
	try {
		// Verify ownership first
		const [formDetails] = await pool.query(
			"SELECT user_id, name FROM forms WHERE uuid = ?",
			[formUuid]
		);
		if (formDetails.length === 0) {
			return res.redirect(
				"/dashboard?errorMessage=" + encodeURIComponent("Form not found.")
			);
		}
		if (formDetails[0].user_id !== req.user.id) {
			return res.redirect(
				"/dashboard?errorMessage=" +
					encodeURIComponent("You do not have permission to delete this form.")
			);
		}

		// Perform deletion. Assuming ON DELETE CASCADE is set up for submissions.
		// If not, delete submissions explicitly first: await pool.query("DELETE FROM submissions WHERE form_uuid = ?", [formUuid]);
		const [deleteResult] = await pool.query(
			"DELETE FROM forms WHERE uuid = ? AND user_id = ?",
			[formUuid, req.user.id]
		);

		if (deleteResult.affectedRows > 0) {
			console.log(
				`Form permanently deleted: ${formDetails[0].name} (UUID: ${formUuid}) by user ${req.user.id}`
			);
			res.redirect(
				"/dashboard?successMessage=" +
					encodeURIComponent(
						`Form '${formDetails[0].name}' and its submissions deleted successfully.`
					)
			);
		} else {
			res.redirect(
				"/dashboard?errorMessage=" +
					encodeURIComponent(
						"Failed to delete form. It might have already been deleted."
					)
			);
		}
	} catch (error) {
		console.error("Error deleting form:", formUuid, error);
		res.redirect(
			"/dashboard?errorMessage=" +
				encodeURIComponent("An error occurred while deleting the form.")
		);
	}
});

// POST /dashboard/submissions/delete/:submissionId - Delete a specific submission
router.post("/submissions/delete/:submissionId", async (req, res) => {
	const { submissionId } = req.params;
	const { formUuidForRedirect } = req.body; // Get this from the form body for redirect

	if (!formUuidForRedirect) {
		console.error(
			"formUuidForRedirect not provided for submission deletion redirect"
		);
		return res.redirect(
			"/dashboard?errorMessage=" +
				encodeURIComponent(
					"Could not determine where to redirect after deletion."
				)
		);
	}

	try {
		// First, verify the user owns the form to which the submission belongs
		const [submissionDetails] = await pool.query(
			`SELECT s.form_uuid, f.user_id 
			 FROM submissions s 
			 JOIN forms f ON s.form_uuid = f.uuid 
			 WHERE s.id = ?`,
			[submissionId]
		);

		if (submissionDetails.length === 0) {
			return res.redirect(
				`/dashboard/submissions/${formUuidForRedirect}?errorMessage=` +
					encodeURIComponent("Submission not found.")
			);
		}

		if (submissionDetails[0].user_id !== req.user.id) {
			return res.redirect(
				`/dashboard/submissions/${formUuidForRedirect}?errorMessage=` +
					encodeURIComponent(
						"You do not have permission to delete this submission."
					)
			);
		}

		// Actual deletion of the submission
		const [deleteResult] = await pool.query(
			"DELETE FROM submissions WHERE id = ?",
			[submissionId]
		);

		if (deleteResult.affectedRows > 0) {
			console.log(
				`Submission ID ${submissionId} deleted by user ${req.user.id}`
			);
			res.redirect(
				`/dashboard/submissions/${formUuidForRedirect}?successMessage=` +
					encodeURIComponent("Submission deleted successfully.")
			);
		} else {
			res.redirect(
				`/dashboard/submissions/${formUuidForRedirect}?errorMessage=` +
					encodeURIComponent(
						"Failed to delete submission. It might have already been deleted."
					)
			);
		}
	} catch (error) {
		console.error(
			"Error deleting submission:",
			submissionId,
			"user:",
			req.user.id,
			error
		);
		res.redirect(
			`/dashboard/submissions/${formUuidForRedirect}?errorMessage=` +
				encodeURIComponent("An error occurred while deleting the submission.")
		);
	}
});

// GET /dashboard/api-keys - Display API key management page
router.get("/api-keys", async (req, res) => {
	try {
		const [keys] = await pool.query(
			"SELECT uuid, key_name, api_key_identifier, created_at, last_used_at, expires_at FROM api_keys WHERE user_id = ? ORDER BY created_at DESC",
			[req.user.id]
		);
		res.render("dashboard", {
			user: req.user,
			view: "api_keys",
			pageTitle: "API Keys",
			apiKeys: keys,
			appUrl: `${req.protocol}://${req.get("host")}`,
			// For displaying a newly generated key (one-time)
			newlyGeneratedApiKey: req.session.newlyGeneratedApiKey,
			newlyGeneratedApiKeyName: req.session.newlyGeneratedApiKeyName,
		});
		// Clear the newly generated key from session after displaying it once
		if (req.session.newlyGeneratedApiKey) {
			delete req.session.newlyGeneratedApiKey;
			delete req.session.newlyGeneratedApiKeyName;
		}
	} catch (error) {
		console.error("Error fetching API keys for user:", req.user.id, error);
		res.render("dashboard", {
			user: req.user,
			view: "api_keys",
			pageTitle: "API Keys",
			apiKeys: [],
			error: "Could not load your API keys at this time.",
			appUrl: `${req.protocol}://${req.get("host")}`,
		});
	}
});

// POST /dashboard/api-keys/generate - Generate a new API key
router.post("/api-keys/generate", async (req, res) => {
	const { keyName } = req.body;
	if (!keyName || keyName.trim() === "") {
		return res.redirect(
			"/dashboard/api-keys?errorMessage=Key name cannot be empty."
		);
	}

	try {
		const { fullApiKey, identifier, secret } = generateApiKeyParts();
		const hashedSecret = await hashApiKeySecret(secret);
		const newApiKeyUuid = uuidv4();

		await pool.query(
			"INSERT INTO api_keys (uuid, user_id, key_name, api_key_identifier, hashed_api_key_secret) VALUES (?, ?, ?, ?, ?)",
			[newApiKeyUuid, req.user.id, keyName.trim(), identifier, hashedSecret]
		);

		console.log(
			`API Key generated for user ${req.user.id}: Name: ${keyName.trim()}, Identifier: ${identifier}`
		);

		// Store the full API key in session to display it ONCE to the user
		// This is a common pattern as the full key should not be retrievable again.
		req.session.newlyGeneratedApiKey = fullApiKey;
		req.session.newlyGeneratedApiKeyName = keyName.trim();

		res.redirect(
			"/dashboard/api-keys?successMessage=API Key generated successfully! Make sure to copy it now, you won\'t see it again."
		);
	} catch (error) {
		console.error("Error generating API key for user:", req.user.id, error);
		// Check for unique constraint violation on api_key_identifier (rare, but possible)
		if (error.code === "ER_DUP_ENTRY") {
			return res.redirect(
				"/dashboard/api-keys?errorMessage=Failed to generate key due to a conflict. Please try again."
			);
		}
		res.redirect(
			"/dashboard/api-keys?errorMessage=Error generating API key. Please try again."
		);
	}
});

// POST /dashboard/api-keys/:apiKeyUuid/revoke - Revoke (delete) an API key
router.post("/api-keys/:apiKeyUuid/revoke", async (req, res) => {
	const { apiKeyUuid } = req.params;
	try {
		const [keyDetails] = await pool.query(
			"SELECT user_id, key_name FROM api_keys WHERE uuid = ? AND user_id = ?",
			[apiKeyUuid, req.user.id]
		);

		if (keyDetails.length === 0) {
			return res.redirect(
				"/dashboard/api-keys?errorMessage=API Key not found or you do not have permission to revoke it."
			);
		}

		await pool.query("DELETE FROM api_keys WHERE uuid = ? AND user_id = ?", [
			apiKeyUuid,
			req.user.id,
		]);

		console.log(
			`API Key revoked: UUID ${apiKeyUuid}, Name: ${keyDetails[0].key_name} by user ${req.user.id}`
		);
		res.redirect(
			"/dashboard/api-keys?successMessage=API Key revoked successfully."
		);
	} catch (error) {
		console.error(
			"Error revoking API key:",
			apiKeyUuid,
			"user:",
			req.user.id,
			error
		);
		res.redirect(
			"/dashboard/api-keys?errorMessage=Error revoking API key. Please try again."
		);
	}
});

module.exports = router;
