const jwt = require("jsonwebtoken");
const { v4: uuidv4 } = require("uuid");
const User = require("../models/User");

class JWTService {
	constructor() {
		this.secret = process.env.JWT_SECRET;
		this.issuer = process.env.JWT_ISSUER || "formies";
		this.audience = process.env.JWT_AUDIENCE || "formies-users";
		this.accessTokenExpiry = process.env.JWT_ACCESS_EXPIRY || "15m";
		this.refreshTokenExpiry = process.env.JWT_REFRESH_EXPIRY || "7d";

		if (!this.secret) {
			throw new Error("JWT_SECRET environment variable is required");
		}
	}

	// Generate access token
	generateAccessToken(user, sessionInfo = {}) {
		const jti = uuidv4(); // JWT ID for token tracking
		const payload = {
			sub: user.id, // Subject (user ID)
			email: user.email,
			role: user.role,
			jti: jti,
			type: "access",
		};

		const options = {
			issuer: this.issuer,
			audience: this.audience,
			expiresIn: this.accessTokenExpiry,
		};

		const token = jwt.sign(payload, this.secret, options);
		const decoded = jwt.decode(token);

		// Save session for token tracking
		const expiresAt = new Date(decoded.exp * 1000);
		User.saveSession(
			user.id,
			jti,
			expiresAt,
			sessionInfo.userAgent,
			sessionInfo.ipAddress
		).catch(console.error);

		return {
			token,
			expiresAt,
			jti,
		};
	}

	// Generate refresh token
	generateRefreshToken(user, sessionInfo = {}) {
		const jti = uuidv4();
		const payload = {
			sub: user.id,
			jti: jti,
			type: "refresh",
		};

		const options = {
			issuer: this.issuer,
			audience: this.audience,
			expiresIn: this.refreshTokenExpiry,
		};

		const token = jwt.sign(payload, this.secret, options);
		const decoded = jwt.decode(token);

		// Save session for token tracking
		const expiresAt = new Date(decoded.exp * 1000);
		User.saveSession(
			user.id,
			jti,
			expiresAt,
			sessionInfo.userAgent,
			sessionInfo.ipAddress
		).catch(console.error);

		return {
			token,
			expiresAt,
			jti,
		};
	}

	// Generate token pair (access + refresh)
	generateTokenPair(user, sessionInfo = {}) {
		const accessToken = this.generateAccessToken(user, sessionInfo);
		const refreshToken = this.generateRefreshToken(user, sessionInfo);

		return {
			accessToken: accessToken.token,
			refreshToken: refreshToken.token,
			accessTokenExpiresAt: accessToken.expiresAt,
			refreshTokenExpiresAt: refreshToken.expiresAt,
			tokenType: "Bearer",
		};
	}

	// Verify and decode token
	verifyToken(token, tokenType = "access") {
		try {
			const options = {
				issuer: this.issuer,
				audience: this.audience,
			};

			const decoded = jwt.verify(token, this.secret, options);

			// Check token type
			if (decoded.type !== tokenType) {
				throw new Error(`Invalid token type. Expected ${tokenType}`);
			}

			return decoded;
		} catch (error) {
			if (error.name === "TokenExpiredError") {
				throw new Error("Token has expired");
			} else if (error.name === "JsonWebTokenError") {
				throw new Error("Invalid token");
			} else if (error.name === "NotBeforeError") {
				throw new Error("Token not active yet");
			}
			throw error;
		}
	}

	// Refresh access token using refresh token
	async refreshAccessToken(refreshToken, sessionInfo = {}) {
		try {
			// Verify refresh token
			const decoded = this.verifyToken(refreshToken, "refresh");

			// Check if token is blacklisted
			const isBlacklisted = await User.isTokenBlacklisted(decoded.jti);
			if (isBlacklisted) {
				throw new Error("Refresh token has been revoked");
			}

			// Get user
			const user = await User.findById(decoded.sub);
			if (!user || !user.is_active) {
				throw new Error("User not found or inactive");
			}

			// Generate new access token
			const newAccessToken = this.generateAccessToken(user, sessionInfo);

			return {
				accessToken: newAccessToken.token,
				accessTokenExpiresAt: newAccessToken.expiresAt,
				tokenType: "Bearer",
			};
		} catch (error) {
			throw error;
		}
	}

	// Revoke token (add to blacklist)
	async revokeToken(token) {
		try {
			const decoded = jwt.decode(token);
			if (!decoded || !decoded.jti) {
				throw new Error("Invalid token format");
			}

			await User.revokeSession(decoded.jti);
			return true;
		} catch (error) {
			console.error("Error revoking token:", error);
			return false;
		}
	}

	// Revoke all user tokens
	async revokeAllUserTokens(userId) {
		try {
			const revokedCount = await User.revokeAllUserSessions(userId);
			return revokedCount;
		} catch (error) {
			console.error("Error revoking all user tokens:", error);
			return 0;
		}
	}

	// Revoke all user tokens except one
	async revokeAllUserTokensExcept(userId, exceptJti) {
		try {
			const revokedCount = await User.revokeAllUserSessionsExcept(
				userId,
				exceptJti
			);
			return revokedCount;
		} catch (error) {
			console.error("Error revoking user tokens:", error);
			return 0;
		}
	}

	// Extract token from Authorization header
	extractTokenFromHeader(authHeader) {
		if (!authHeader) {
			return null;
		}

		const parts = authHeader.split(" ");
		if (parts.length !== 2 || parts[0] !== "Bearer") {
			return null;
		}

		return parts[1];
	}

	// Get token info without verification
	getTokenInfo(token) {
		try {
			return jwt.decode(token);
		} catch (error) {
			return null;
		}
	}

	// Check if token is expired (without verifying signature)
	isTokenExpired(token) {
		const decoded = this.getTokenInfo(token);
		if (!decoded || !decoded.exp) {
			return true;
		}

		return Date.now() >= decoded.exp * 1000;
	}

	// Cleanup expired sessions (call this periodically)
	async cleanupExpiredSessions() {
		try {
			const cleanedCount = await User.cleanupExpiredSessions();
			console.log(`Cleaned up ${cleanedCount} expired sessions`);
			return cleanedCount;
		} catch (error) {
			console.error("Error cleaning up expired sessions:", error);
			return 0;
		}
	}

	// Get current session information
	async getCurrentSession(token) {
		try {
			const decoded = this.verifyToken(token);
			const session = await User.getSessionByJti(decoded.jti);

			if (!session) {
				throw new Error("Session not found");
			}

			return {
				jti: session.token_jti,
				userAgent: session.user_agent,
				ipAddress: session.ip_address,
				createdAt: session.created_at,
				expiresAt: session.expires_at,
			};
		} catch (error) {
			throw error;
		}
	}
}

module.exports = new JWTService();
