// Native fetch is available in Node.js 18+ and doesn't need to be imported
// const logger = require("../../config/logger"); // Adjust path as needed

const RECAPTCHA_V2_SECRET_KEY = process.env.RECAPTCHA_V2_SECRET_KEY;
const GOOGLE_RECAPTCHA_VERIFY_URL =
	"https://www.google.com/recaptcha/api/siteverify";

/**
 * Verifies a Google reCAPTCHA v2 response.
 * @param {string} recaptchaToken - The g-recaptcha-response token from the client.
 * @param {string} [clientIp] - Optional. The user's IP address.
 * @returns {Promise<boolean>} - True if verification is successful, false otherwise.
 */
async function verifyRecaptchaV2(recaptchaToken, clientIp) {
	if (!RECAPTCHA_V2_SECRET_KEY) {
		console.warn(
			"RECAPTCHA_V2_SECRET_KEY is not set. Skipping reCAPTCHA verification. THIS IS INSECURE FOR PRODUCTION."
		);
		// In a real scenario, you might want to fail open or closed based on policy
		// For now, let's assume if it's not set, we can't verify, so effectively it fails if meant to be checked.
		// However, the calling route will decide if reCAPTCHA is mandatory.
		return false; // Or true if you want to bypass if not configured, though less secure.
	}

	if (!recaptchaToken) {
		console.warn("No reCAPTCHA token provided by client.");
		return false;
	}

	const verificationUrl = `${GOOGLE_RECAPTCHA_VERIFY_URL}?secret=${RECAPTCHA_V2_SECRET_KEY}&response=${recaptchaToken}`;
	// Add remoteip if provided
	const finalUrl = clientIp
		? `${verificationUrl}&remoteip=${clientIp}`
		: verificationUrl;

	try {
		const response = await fetch(finalUrl, { method: "POST" });
		const data = await response.json();

		if (data.success) {
			console.info("reCAPTCHA verification successful.");
			return true;
		} else {
			console.warn(
				"reCAPTCHA verification failed.",
				data["error-codes"] || "No error codes"
			);
			return false;
		}
	} catch (error) {
		console.error("Error during reCAPTCHA verification request:", error);
		return false;
	}
}

module.exports = { verifyRecaptchaV2 };
